#include <msp430.h> 
#include <stdint.h>

#include "RF1A.h"
#include "RF_Connection.h"
#include "PMM.h"
#include "ports.h"
#include "RFconfig.h"
#include "led_commands.h"

#define BUFFER_SIZE		10	// maximum size of data sent by rf


#define PWM_PERIOD		255
#define DELAY 100 //300us  
#define DELAY_RSSI 10 //30us

#define START_FREQ_CHAN 15
#define STOP_FREQ_CHAN 50     //scanning 863-870Mhz with 200khz steps
#define RSSI_OFFSET 74 //cf datasheet

#define NUM_JAM_RAMP 10


volatile uint8_t buffer[BUFFER_SIZE];
volatile uint8_t temp[NB_BANGLES][2];
volatile uint8_t fin[NB_BANGLES][NB_BANGLES-1];


uint8_t RxBuffer[BUFFER_SIZE];
uint8_t RxBufferLength=sizeof(RxBuffer);



unsigned char chan_jam=0x00;
unsigned char led2=0x10;
unsigned char led1=0x10;



void packet_received(void)
{
  RxBufferLength = ReadSingleReg(RXBYTES);
  ReadBurstReg(RF_RXFIFORD, RxBuffer, RxBufferLength);
}


void freq_scan(void)
{
  unsigned char rssi;
  long rssi_dbm;
  unsigned char chan_scan=START_FREQ_CHAN;
  long rssi_max=0; //scan will take communication above 0dbm
  set_color(led1, 0x00, 0x00, 0x00, led2, 0x00); //led2 blue while scanning
  
  ChangeChannel(chan_scan);
  ReceiveOn();    //enable RX
  while((Strobe(RF_SNOP) & 0x70) != 0x10); //make sure RX is ready
  while(chan_scan<STOP_FREQ_CHAN)
  {
    ChangeChannel(chan_scan);
    __delay_cycles(DELAY_RSSI);       //wait for rssi to be updated
    rssi = ReadSingleReg(RSSI);  //define in cc430f5137.h RSSI=0x34 corresponding to addr of rssi value
    if (rssi >= 128) 
    {
      rssi_dbm = (long)((long)( rssi - 256) / 2) - RSSI_OFFSET; 
    }
    else 
    {
      rssi_dbm = (rssi / 2) - RSSI_OFFSET; 
    }
    if(rssi_dbm>rssi_max)
    {
      rssi_max=rssi_dbm;
      chan_jam=chan_scan-10; //start jamming 2Mhz below max dbm received
    }
    chan_scan++;
    Strobe( RF_SFRX); //flush the RX FIFO
  }
  set_color(led1, 0x00, 0x00, 0x00, 0x00, 0x00);
}



void send_cmd(void)
{
  buffer[0] = 0xFD;
  buffer[1] = 0xAA;
  buffer[2] = 0xFF;
  buffer[3] = 0x11;
  buffer[4] = 0xFF;
  buffer[5] = 0x66;
  buffer[6] = 0xFF;
  buffer[7] = 0x33;
  buffer[8] = 0x1F;
  buffer[9] = 0xF8;
  rf_transmit((uint8_t *)buffer, 2);
}

int main(void)
{
  unsigned char chan_main=0x00;
  port_mapping();
  WDTCTL = WDTPW + WDTHOLD;   // Stop WDT
  rf_init(); //reset, setVcore & patable
  _BIS_SR(GIE);
  set_color(led1, 0x00, 0x00, 0x00, 0x00, 0x00); //led1 on if cc430 powered
  int i;
  
  while(1)
  {
    freq_scan();
    ReceiveOff();
    chan_main=chan_jam;
    if (chan_main != 0)  //jam only if a communication xas detected with freq_scan
    {
      for(i=0 ; i<(20*NUM_JAM_RAMP) ; i++) //ramp start 2Mhz below chan_jam and stop 2Mhz above
      {
	ChangeChannel(chan_main);
	if (chan_main>= (chan_jam+20))
	{
	  send_cmd();
	  chan_main=0x00;
	  led2=led2^0x10;
	  set_color(led1, 0x00, 0x00, led2, 0x00, 0x00); //led2 blink red every two ramp of jamming
	}
	else
	{
	  send_cmd();
	  chan_main++;
	}
      }
    }
    chan_main=0x00;  
    __delay_cycles(DELAY);
  }
  return 0;
}

